
/* V 25.10.23 */

#include <EEPROM.h>
#include <OneWire.h>
#include <Wire.h>
#include <MsTimer2.h>
#include <Encoder.h>
#include <PID_v1.h>
#include <U8glib.h>
#include "GyverPWM.h"
#include "rus6x10.h",
#include "Rus_10x20.h",

U8GLIB_ST7920_128X64_4X u8g(13, 11, 10);

double Setpoint_F, Input_F, Output_F, Setpoint_C, Input_C, Output_C;
double Kp_F = 0.5, Ki_F = 3.0, Kd_F = 0.1;
double Kp_C = 0.5, Ki_C = 205.0, Kd_C = 0.5;
PID PID_FAN(&Input_F, &Output_F, &Setpoint_F, Kp_F, Ki_F, Kd_F, DIRECT);
PID PID_CUR(&Input_C, &Output_C, &Setpoint_C, Kp_C, Ki_C, Kd_C, DIRECT);
/*-----------------------------------------------------------------------------------------------------*/
Encoder myEnc(5, 3);
#define SB_Enc            4
long oldPosition        = 0;
#define KEY_Start         12
byte SB_Touch           = 0;
/*-----------------------------------------------------------------------------------------------------*/
float Volt[]      = {0, 0};
float Curr[]      = {0.00, 0.00, 1.5};
float Curr_Var    = 7.17;
byte Zar          = 0;
byte Select       = 0;
byte Level        = 255;

byte Second[]     = {0};
byte Minute[]     = {0};
byte Hour[]       = {0}; 
int  Sec[]        = {0, 0, 0, 0, 0};
byte Dw           = 0;
/*-----------------------------------------------------------------------------------------------------*/
OneWire DS_BP(8);                                     // Pin подключения датчика температуры

#define POWER_MODE  0                                 // Режим питания, 0 - внешнее, 1 - паразитное
boolean FlagSensReady_BP;                             // Признак готовности данных с датчика
byte  bufData_BP[9];                                  // Буфер данных
int   TimeCount;                                      // Счетчик времени измерения
byte  Present_BP          = 0;                        // Сигнал наличия термодатчика
byte  Flag_Cels           = 0;

byte  Cels[]              = {0, 30, 40, 50, 60, 70, 80, 90, 100};
byte  ii[]                = {0, 0, 10, 20, 30, 40, 50, 60, 70};
byte  Cls[]               = {0, 0};
/*-------------------------------------------------------------------------------------------------------
                                            Переменные контроля напряжения
-------------------------------------------------------------------------------------------------------*/
const float R1              = 51000;                  // Резисторы делителя напряжения
const float R2              = 5100;

const float Typ_Vbg = 1.105; // 1.0 -- 1.2            // Эту константу необходимо откалибровать индивидуально
float Typ_Vbg_Old;

float V                     = 0.0;
float Vcc                   = 0.0;
float Max_Volt              = 0.0;
/*-----------------------------------------------------------------------------------------------------*/
#define PWM_OUT               9
#define Fan_PWM               6
#define Speed                 2
byte  Frequens[]            = {0, 0};
byte  Freq[]                = {0, 0, 0};
byte  Adress                = 2;
byte  Temp[2];
/*-----------------------------------------------------------------------------------------------------*/
#define CLR(x, y) (x &= (~(1 << y))) 
#define SET(x, y) (x |= (1 << y)) 
int   Tmp_L                 = 1;
byte  Sec_1[]               = {0, 0};
byte  PWM_ON                = 127;
byte  PWM_OFF               = 127;
byte  PWM_P                 = 0;
void ISR_F() {Frequens[1]++;}
/*=======================================================================================================
                                              Чтение EEPROM
=======================================================================================================*/
void EEPROM_Read()
{
  for (int i = 0; i <= Adress; i++)
  {
    Temp[i] = EEPROM.read(i);                // считываем значение по текущему адресу EEPROM
     Serial.print(" : ");Serial.print(i); Serial.print(" = "); Serial.print(Temp[i]);
  }
  Level = Temp[1]; Curr[2] = (float)Temp[2] / 10;
  Serial.println(" << READ >> ");
}
/*=======================================================================================================
                                              Запись EEPROM
=======================================================================================================*/
void EEPROM_Write()
{
  Curr[2] *= 10; Serial.print(" : Curr[2] = ");  Serial.print(Curr[2]);
  Temp[1] = Level; Temp[2] = Curr[2];
  for (int i = 0; i <= Adress; i++)
  {
    EEPROM.write(i, Temp[i]);
   Serial.print(" : ");  Serial.print(i); Serial.print(" = "); Serial.print(Temp[i]);
  }
  Serial.println(" << WRITE >> ");
}
/*=======================================================================================================
                                            Инициализация
=======================================================================================================*/
void setup()
{
  Serial.begin(115200);
  Wire.begin();
  
  EEPROM_Read();

  pinMode(Speed, INPUT);
  pinMode(PWM_OUT, OUTPUT);
  analogWrite(PWM_OUT, 255);
  pinMode(SB_Enc, INPUT_PULLUP);
  pinMode(Fan_PWM, OUTPUT);
  digitalWrite(Fan_PWM, HIGH);
  
  u8g.begin();
  u8g.setColorIndex(7);                               // Определяем закрасить (1) или нет (0)
/*-----------------------------------------------------------------------------------------------------*/
  PID_FAN.SetTunings(Kp_F, Ki_F, Kd_F);
  PID_FAN.SetMode(AUTOMATIC);
  PID_FAN.SetOutputLimits(0, 255);  
  PID_FAN.Compute();
  PID_CUR.SetTunings(Kp_C, Ki_C, Kd_C);
  PID_CUR.SetMode(AUTOMATIC);
  PID_CUR.SetOutputLimits(0, 255);  
  PID_CUR.Compute();
/*-----------------------------------------------------------------------------------------------------*/
  analogReference(DEFAULT);                             // DEFAULT INTERNAL использовать Vcc как AREF
  delay(100);
  Vcc = Read_Vcc();
  Max_Volt = Vcc / (R2 / (R1 + R2));
  Serial.print(" <<   Max_Volt   =  "); Serial.print(Max_Volt); Serial.println("   >> ");
  analogWrite(0, 0);
  Read_Volt();
/*-----------------------------------------------------------------------------------------------------*/
  PWM_frequency(9, 5000, FAST_PWM);

  attachInterrupt(0, ISR_F, RISING);

  MsTimer2::set(1, Timing_ISR);                       // Задаем период прерывания по таймеру 1 мс
  MsTimer2::start();                                  // Разрешаем прерывание по таймеру
}
/*=======================================================================================================
                                              Главный цикл
=======================================================================================================*/
void loop() 
{
  if (Flag_Cels == 1) Read_DS18B20();
  Read_Volt();
  Indic();
  for (byte i = 8; i >= 1; i--)
  {
    if (Cels[0] <= Cels[i]) Input_F = Cels[0] + ii[i];
  }
  Setpoint_F = Frequens[0];
  PID_FAN.Compute();
  analogWrite(Fan_PWM, Output_F);
}
/*=======================================================================================================
                                              Encoder
=======================================================================================================*/
void Encode()
{
  long newPosition = myEnc.read() / 2;
  if (oldPosition != newPosition) 
  {
    if (oldPosition < newPosition)
    {
      if (Select == 0) 
      {
        if (Level <= 254) Level++;
        if (PWM_OFF >= 1) PWM_OFF--;
        if (PWM_ON  <= 254) PWM_ON++;
      }
      else {if (Curr[2] >= 0.1) Curr[2] -= 0.1; Sec[3] = 0;}
    }
    if (oldPosition > newPosition)
    {
      if (Select == 0) 
      {
        if (Level >= 1) Level--;
        if (PWM_ON >= 1) PWM_ON--;
        if (PWM_OFF  <= 254) PWM_OFF++;
      }
      else {if (Curr[2] <= 10.0) Curr[2] += 0.1; Sec[3] = 0;}
    }
    oldPosition = newPosition;
  }
/*-----------------------------------------------------------------------------------------------------*/
    if (digitalRead(SB_Enc) == 0)
    {
      switch (Select)
      {
        case 0: Select = 1; Sec[3] = 0; break;
        case 1: Select = 0; break;
      }
      SB_Touch = 0; Sec[2] = 0;
      while (SB_Touch == 0)
      {
        if (digitalRead(SB_Enc) == 0) SB_Touch = 0; else SB_Touch = 1;
        Sec[2]++;
        if (Sec[2] >= 180)
        {
          Sec[2] = 0;
          //Write = 1; 
          EEPROM_Write(); EEPROM_Read();
          Select = 1; 
          SB_Touch = 1;
        }
        delay(10);        
      }
    }
}
/*=======================================================================================================
                                              Чтение температуры
=======================================================================================================*/
void Read_DS18B20()
{
  if (FlagSensReady_BP == true)
  {
    FlagSensReady_BP = false;                                   // Данные готовы

    if (OneWire::crc8(bufData_BP, 8) == bufData_BP[8])          // Проверка CRC данные правильные
    {
      Cels[0] = ((float)((int)bufData_BP[0] | (((int)bufData_BP[1]) << 8)) * 0.0625 + 0.03125);
    }
  }
  Flag_Cels = 0;
}
/*=====================================================================================================*/
